-module(recursion).
-compile(export_all).
 
%% EXAMPLE: Factorial with different syntax

% with GUARDS and PATTERN MATCHING
fac(0) -> 1;
fac(N) when N > 0  -> N*fac(N-1);
fac(_) -> 0.

% with IFs
fact(N) -> if N == 0 -> 1
	        ; N > 0  -> N*fact(N-1)
	        ; true -> 0 
	       end .

% TAIL RECURSION
tail_fac(N) -> tail_fac(N,1).
 
tail_fac(0,Acc) -> Acc;
tail_fac(N,Acc) when N > 0 -> tail_fac(N-1,N*Acc);
tail_fac(_,_) -> 0.
% recursion:tail_fac(-2).
% recursion:tail_fac(30).

%% EXAMPLE: permutations

perms([]) -> [[]];
perms(L)  -> [[H|T] || H <- L, T <- perms(L--[H])].
% recursion:perms("abc").

%% EXAMPLE: length of a list

len([]) -> 0;
len([_|T]) -> 1 + len(T).

tail_len(L) -> tail_len(L,0).

tail_len([],Acc) -> Acc;
tail_len([_|T],Acc) -> tail_len(T,Acc+1).

%% EXAMPLE: replicate

replicate(0,_) -> [];
replicate(N,Term) when N > 0 -> [Term|replicate(N-1,Term)].

tail_replicate(N,Term) -> tail_replicate(N,Term,[]).
 
tail_replicate(0,_,List) -> List;
tail_replicate(N,Term,List) when N > 0 -> tail_replicate(N-1, Term, [Term|List]).

%% EXAMPLE: reverse

reverse([]) -> [];
reverse([H|T]) -> reverse(T)++[H].
% costs too much!!

tail_reverse(L) -> tail_reverse(L,[]).
 
tail_reverse([],Acc) -> Acc;
tail_reverse([H|T],Acc) -> tail_reverse(T, [H|Acc]).


